using ImmichToSlideshow.Services;
using Microsoft.AspNetCore.Mvc;

namespace ImmichToSlideshow.Controllers;

[ApiController]
[Route("api/v1/[controller]")]
public class AssetsController(AssetService assetService) : ControllerBase
{

    private readonly string _ContentType = "application/json";
    private readonly AssetService _AssetService = assetService;

    [HttpGet("columns")]
    public IActionResult GetColumns() =>
        Content(_AssetService.GetColumns(), _ContentType);

    [HttpGet("owner-ids")]
    public IActionResult GetOwnerIds() =>
        Content(_AssetService.GetOwnerIds(), _ContentType);

    [HttpGet("{ownerId:guid}")]
    public IActionResult Get(Guid ownerId) =>
        Content(_AssetService.GetAssets(ownerId), _ContentType);

    [HttpGet("{ownerId:guid}/random-paths")]
    public IActionResult GetRandomPaths(Guid ownerId) =>
        Ok(_AssetService.GetRandomPaths(ownerId));

    [HttpGet("{ownerId:guid}/archived-tag")]
    public IActionResult GetArchivedTag(Guid ownerId) =>
        Content(_AssetService.GetArchivedTag(ownerId), _ContentType);

    [HttpGet("{ownerId:guid}/save-random-paths")]
    public IActionResult SaveRandomPaths(Guid ownerId) =>
        Content(_AssetService.SaveRandomPaths(ownerId), _ContentType);

    [HttpGet("{ownerId:guid}/sync-immich")]
    public IActionResult SyncImmich(Guid ownerId) =>
        Ok(_AssetService.SyncImmich(ownerId));

    [HttpGet("{ownerId:guid}/set-archive-immich")]
    public IActionResult SetArchiveImmich(Guid ownerId) =>
        Ok(_AssetService.SetArchiveImmich(ownerId));

}