using System.Text.Json;
using System.Text.Json.Serialization;

namespace ImmichToSlideshow.Models.Binder;

public class AppSettings
{

    public int? AddDays { get; set; }
    public string? ArchivedTag { get; set; }
    public string? Company { get; set; }
    public string? ConnectionString { get; set; }
    public string? CurrentCommit { get; set; }
    public string? CurrentResultsDirectory { get; set; }
    public string[]? FilterTags { get; set; }
    public string? ImmichUploadDirectory { get; set; }
    public string? RandomResultsDirectory { get; set; }
    public string? SyncDirectory { get; set; }
    public string? URLs { get; set; }
    public string[]? WithOrigins { get; set; }
    public string? WorkingDirectoryName { get; set; }

    public override string ToString()
    {
        string result = JsonSerializer.Serialize(this, BinderAppSettingsSourceGenerationContext.Default.AppSettings);
        return result;
    }

    private static void PreVerify(IConfigurationRoot configurationRoot, AppSettings? appSettings)
    {
        if (appSettings?.Company is null)
        {
            List<string> paths = [];
            foreach (IConfigurationProvider configurationProvider in configurationRoot.Providers)
            {
                if (configurationProvider is not Microsoft.Extensions.Configuration.Json.JsonConfigurationProvider jsonConfigurationProvider)
                    continue;
                if (jsonConfigurationProvider.Source.FileProvider is not Microsoft.Extensions.FileProviders.PhysicalFileProvider physicalFileProvider)
                    continue;
                paths.Add(physicalFileProvider.Root);
            }
            throw new NotSupportedException($"Not found!{Environment.NewLine}{string.Join(Environment.NewLine, paths.Distinct())}");
        }
    }

    private static Models.AppSettings Get(AppSettings? appSettings)
    {
        Models.AppSettings result;
        if (appSettings?.AddDays is null) throw new NullReferenceException(nameof(appSettings.AddDays));
        if (appSettings?.ArchivedTag is null) throw new NullReferenceException(nameof(appSettings.ArchivedTag));
        if (appSettings?.Company is null) throw new NullReferenceException(nameof(appSettings.Company));
        if (appSettings?.ConnectionString is null) throw new NullReferenceException(nameof(appSettings.ConnectionString));
        if (appSettings?.CurrentCommit is null) throw new NullReferenceException(nameof(appSettings.CurrentCommit));
        if (appSettings?.CurrentResultsDirectory is null) throw new NullReferenceException(nameof(appSettings.CurrentResultsDirectory));
        if (appSettings?.FilterTags is null) throw new NullReferenceException(nameof(appSettings.FilterTags));
        if (appSettings?.ImmichUploadDirectory is null) throw new NullReferenceException(nameof(appSettings.ImmichUploadDirectory));
        if (appSettings?.RandomResultsDirectory is null) throw new NullReferenceException(nameof(appSettings.RandomResultsDirectory));
        if (appSettings?.SyncDirectory is null) throw new NullReferenceException(nameof(appSettings.SyncDirectory));
        if (appSettings?.URLs is null) throw new NullReferenceException(nameof(appSettings.URLs));
        if (appSettings?.WithOrigins is null) throw new NullReferenceException(nameof(appSettings.WithOrigins));
        if (appSettings?.WorkingDirectoryName is null) throw new NullReferenceException(nameof(appSettings.WorkingDirectoryName));
        result = new(appSettings.AddDays.Value,
                     appSettings.ArchivedTag,
                     appSettings.Company,
                     appSettings.ConnectionString,
                     appSettings.CurrentCommit,
                     appSettings.CurrentResultsDirectory,
                     appSettings.FilterTags,
                     appSettings.ImmichUploadDirectory,
                     appSettings.RandomResultsDirectory,
                     appSettings.SyncDirectory,
                     appSettings.URLs,
                     appSettings.WithOrigins,
                     appSettings.WorkingDirectoryName);
        return result;
    }

    public static Models.AppSettings Get(IConfigurationRoot configurationRoot)
    {
        Models.AppSettings result;
#pragma warning disable IL3050, IL2026
        AppSettings? appSettings = configurationRoot.Get<AppSettings>();
#pragma warning restore IL3050, IL2026
        PreVerify(configurationRoot, appSettings);
        result = Get(appSettings);
        return result;
    }

}

[JsonSourceGenerationOptions(WriteIndented = true)]
[JsonSerializable(typeof(AppSettings))]
internal partial class BinderAppSettingsSourceGenerationContext : JsonSerializerContext
{
}